MODULE FractalNetObj_T_v1 EXPORTS FractalNetObj, FractalNetObj_T_v1;

IMPORT Thread, NetObj, StubLib, FractalNetObj, Rd, Wr;
CONST Protocol: StubLib.StubProtocol = 1;

TYPE
      Methods = {returnResults, requestFractal};
      ReturnCodes = {OK};

  PROCEDURE Surrogate_requestFractal(self: FractalNetObj.T)
      : FractalNetObj.Fractal RAISES {NetObj.Error, Thread.Alerted} =

    VAR reuse := FALSE;
        rep: StubLib.DataRep;
        c: StubLib.Conn;
        dataPresent: BOOLEAN; <* NOWARN *>
        res: FractalNetObj.Fractal;

    BEGIN
      TRY
        c := StubLib.StartCall(self, Protocol);
        TRY
          StubLib.OutInt32(c, ORD(Methods.requestFractal));
          rep := StubLib.AwaitResult(c);
          CASE StubLib.InInt32(c, rep) OF
          | ORD(ReturnCodes.OK) =>
            res := StubLib.InRef(c, rep, TYPECODE(FractalNetObj.Fractal));
            reuse := TRUE;
          ELSE
            StubLib.RaiseUnmarshalFailure();
          END
        FINALLY
          StubLib.EndCall(c, reuse);
        END;

      EXCEPT
      | Rd.Failure(ec) => StubLib.RaiseCommFailure(ec);
      | Wr.Failure(ec) => StubLib.RaiseCommFailure(ec);
      END;
      RETURN res;
    END Surrogate_requestFractal;

  PROCEDURE Surrogate_returnResults(
      self: FractalNetObj.T;
      f_arg: FractalNetObj.FractalParams;
      pixmap_arg: FractalNetObj.PixMap) RAISES {NetObj.Error,
      Thread.Alerted} =

    VAR reuse := FALSE;
        rep: StubLib.DataRep;
        c: StubLib.Conn;
        dataPresent: BOOLEAN; <* NOWARN *>

    BEGIN
      TRY
        c := StubLib.StartCall(self, Protocol);
        TRY
          StubLib.OutInt32(c, ORD(Methods.returnResults));
          dataPresent := TRUE;
          StubLib.OutBoolean(c, dataPresent);
          IF dataPresent THEN
            StubLib.OutRef(c, f_arg.type);
            StubLib.OutInteger(c, f_arg.sx);
            StubLib.OutInteger(c, f_arg.sy);
            StubLib.OutReal(c, f_arg.xmin);
            StubLib.OutReal(c, f_arg.xmax);
            StubLib.OutReal(c, f_arg.ymin);
            StubLib.OutReal(c, f_arg.ymax);
            StubLib.OutInteger(c, f_arg.maxiter);
            END;
            StubLib.OutRef(c, pixmap_arg);
          rep := StubLib.AwaitResult(c);
          CASE StubLib.InInt32(c, rep) OF
          | ORD(ReturnCodes.OK) =>
            reuse := TRUE;
          ELSE
            StubLib.RaiseUnmarshalFailure();
          END
        FINALLY
          StubLib.EndCall(c, reuse);
        END;

      EXCEPT
      | Rd.Failure(ec) => StubLib.RaiseCommFailure(ec);
      | Wr.Failure(ec) => StubLib.RaiseCommFailure(ec);
      END;
    END Surrogate_returnResults;

PROCEDURE Invoke(
    c: StubLib.Conn;
    obj: NetObj.T;
    rep: StubLib.DataRep;
    stubProt: StubLib.StubProtocol)
    RAISES {NetObj.Error, Rd.Failure,
            Wr.Failure, Thread.Alerted} =
  VAR t := NARROW(obj, FractalNetObj.T);
  BEGIN
    IF stubProt # Protocol THEN StubLib.RaiseUnmarshalFailure() END;
    TRY
      CASE StubLib.InInt32(c, rep) OF
      | ORD(Methods.requestFractal) => Stub_requestFractal(t, c, rep);
      | ORD(Methods.returnResults) => Stub_returnResults(t, c, rep);
      ELSE
        StubLib.RaiseUnmarshalFailure();
      END;
    EXCEPT
    END;
  END Invoke;

PROCEDURE Stub_requestFractal(
    self: FractalNetObj.T;
    c: StubLib.Conn;
    <* NOWARN *> rep: StubLib.DataRep) RAISES {NetObj.Error, Rd.Failure,
    Wr.Failure, Thread.Alerted}=
  VAR res: FractalNetObj.Fractal;
      dataPresent: BOOLEAN <* NOWARN *>;

  BEGIN
    res := self.requestFractal();
    StubLib.StartResult(c);
    StubLib.OutInt32(c, ORD(ReturnCodes.OK));
    StubLib.OutRef(c, res);

  END Stub_requestFractal;

PROCEDURE Stub_returnResults(
    self: FractalNetObj.T;
    c: StubLib.Conn;
    <* NOWARN *> rep: StubLib.DataRep) RAISES {NetObj.Error, Rd.Failure,
    Wr.Failure, Thread.Alerted}=
  VAR f_arg: FractalNetObj.FractalParams;
      pixmap_arg: FractalNetObj.PixMap;
      dataPresent: BOOLEAN <* NOWARN *>;

  BEGIN
    dataPresent := StubLib.InBoolean(c);
    IF dataPresent THEN
      f_arg.type := StubLib.InRef(c, rep, -1);
      f_arg.sx := StubLib.InInteger(c, rep);
      f_arg.sy := StubLib.InInteger(c, rep);
      f_arg.xmin := StubLib.InReal(c, rep);
      f_arg.xmax := StubLib.InReal(c, rep);
      f_arg.ymin := StubLib.InReal(c, rep);
      f_arg.ymax := StubLib.InReal(c, rep);
      f_arg.maxiter := StubLib.InInteger(c, rep);
      END;
      pixmap_arg := StubLib.InRef(c, rep, TYPECODE(FractalNetObj.PixMap));
    self.returnResults(f_arg, pixmap_arg);
    StubLib.StartResult(c);
    StubLib.OutInt32(c, ORD(ReturnCodes.OK));

  END Stub_returnResults;

BEGIN
  StubLib.Register(TYPECODE(FractalNetObj.T), 1, TYPECODE(Surrogate_FractalNetObj_T), Invoke);
END FractalNetObj_T_v1.
