MODULE Client EXPORTS Main;

IMPORT FractalNetObj, NetObj, IP, Fmt, IO, Params, Wr, Stdio, Thread;

PROCEDURE Put (t: TEXT) = 
  <* FATAL Wr.Failure, Thread.Alerted *>
  BEGIN
    IO.Put(t); Wr.Flush(Stdio.stdout); Wr.Flush(Stdio.stderr);
  END Put;

PROCEDURE Compute (f: FractalNetObj.FractalParams): FractalNetObj.PixMap = 
  VAR
    pixmap: FractalNetObj.PixMap := NEW(FractalNetObj.PixMap,f.sx,f.sy);
    dx: REAL := (f.xmax - f.xmin) / FLOAT(f.sx);
    dy: REAL := (f.ymax - f.ymin) / FLOAT(f.sy);
    m,px,py: INTEGER := 0;
    old_x,fx,fy,x: REAL := 0.0;
    y: REAL := f.ymin;
  BEGIN
    WHILE py < f.sy DO
      px := 0;
      x := f.xmin;
      py := py + 1;
      WHILE px < f.sx DO
        px := px + 1;
        fx := 0.0;
        fy := 0.0;
        m := 0;
        REPEAT
          old_x := fx;
          fx := fx*fx - fy*fy + x;
          fy := 2.0*old_x*fy + y;
          m := m + 1;
        UNTIL (((fx*fx + fy*fy) > 4.0) OR (m = f.maxiter));
        IF m # f.maxiter THEN pixmap[px-1][py-1] := m;
        ELSE pixmap[px-1][py-1] := 1;
        END;
        x := x + dx;
      END;
      y := y + dy;
    END;
    RETURN pixmap; 
  END Compute;

VAR
  myFractalNetObj: FractalNetObj.T;
  myFractal: FractalNetObj.Fractal := NIL;
  fractalParams: FractalNetObj.FractalParams;
  pixmap: FractalNetObj.PixMap;
  host: TEXT;
  addr: IP.Address;
  agent: NetObj.Address;
  workUnits: INTEGER := 0;

EXCEPTION Failure;

BEGIN
  TRY
    IF Params.Count # 2 THEN
      Put ("Usage: " & Params.Get (0) & " <host>\n");
      RAISE Failure;
    ELSE
    host := Params.Get(1);
    IF NOT IP.GetHostByName(host, addr) THEN
      Put(Fmt.F("No such host \"%s\"\n", host));
      RAISE Failure;
    ELSE
      agent := NetObj.Locate(host);
    END;
    END;
    
    myFractalNetObj := NetObj.Import ("fractal", agent);
    IF myFractalNetObj = NIL THEN
      Put ("Error: a fractal server is not running on \"" & host & "\"\n");
      RAISE Failure
    END;
    WHILE TRUE DO
      workUnits := workUnits + 1;
      Put ("Requesting work unit " & Fmt.Int(workUnits) & "...\n");
      REPEAT
        myFractal := myFractalNetObj.requestFractal();
      UNTIL myFractal # NIL;
      fractalParams := myFractal.getFractalParameters();
      pixmap := Compute(fractalParams);
      myFractalNetObj.returnResults(fractalParams,pixmap);
    END;
  EXCEPT
  | NetObj.Error => Put ("Netobj.error\n");
  | Failure => (* empty *)
  ELSE Put ("An unknown fatal error occurred.")
  END;
END Client.
