MODULE Server EXPORTS Main;

IMPORT FractalNetObj, NetObj, Atom, IO, Wr, Stdio, Rd, Text, Fmt,Process;
IMPORT Thread,ParseParams;

PROCEDURE Put (t: TEXT) = 
  <* FATAL Wr.Failure, Thread.Alerted *>
  BEGIN
    IO.Put(t); Wr.Flush(Stdio.stdout); Wr.Flush(Stdio.stderr);
  END Put;

TYPE
  FractalImpl = FractalNetObj.Fractal OBJECT
    type: TEXT;
    sx: INTEGER; sy: INTEGER; xmin: REAL; xmax: REAL;
    ymin: REAL; ymax: REAL; maxiter: INTEGER;
  OVERRIDES
    getFractalParameters := GetFractalParameters;
  END;

  FractalNetObjImpl = FractalNetObj.T OBJECT
  OVERRIDES
    requestFractal := RequestFractal;
    returnResults := ReturnResults;
  END;

PROCEDURE GetFractalParameters(f: FractalImpl): FractalNetObj.FractalParams =
  VAR
    fractalParams: FractalNetObj.FractalParams;
  BEGIN
    LOCK mu DO
      fractalParams.sx := f.sx;
      fractalParams.sy := f.sy;
      fractalParams.xmin := f.xmin;
      fractalParams.xmax := f.xmax;
      fractalParams.ymin := f.ymin;
      fractalParams.ymax := f.ymax;
      fractalParams.maxiter := f.maxiter;
      RETURN fractalParams;
    END;
  END GetFractalParameters;

PROCEDURE RequestFractal(self: FractalNetObjImpl): FractalNetObj.Fractal = 
  VAR
    fractal: FractalImpl;
  BEGIN
    LOCK mu DO
      IF curY < sy THEN 
(*        Put ("Dispatching line " & Fmt.Int(curY) & "\n"); *)
        fractal := NEW (FractalImpl, sx := sx, sy := 1, 
                        xmin := xmin, xmax := xmax, 
                        ymin := FLOAT(curY)*((ymax-ymin)/FLOAT(sy))+ymin, 
                        ymax := FLOAT(curY)*((ymax-ymin)/FLOAT(sy))+ymin, 
                        maxiter := maxiter);
        curY := curY + 1;
        RETURN fractal;
      END; 
      RETURN NIL;
    END;
  END RequestFractal;

PROCEDURE validCoords(mouse: FractalNetObj.MouseInfoStar;
                      sx: REAL; sy: REAL): BOOLEAN =
  BEGIN
    IF FLOAT(mouse.x) < 0.0 THEN RETURN FALSE;
    ELSIF FLOAT(mouse.x) > sy THEN RETURN FALSE;
    ELSIF FLOAT(mouse.y) < 0.0 THEN RETURN FALSE;
    ELSIF FLOAT(mouse.y) > sx THEN RETURN FALSE;
    ELSE RETURN TRUE;
    END;
  END validCoords;

PROCEDURE ReturnResults(self: FractalNetObjImpl;
                        f: FractalNetObj.FractalParams;
                        pixmap: FractalNetObj.PixMap) = 
  VAR
    xlow,ylow,xhigh,yhigh: INTEGER;
    xrange,yrange,xjump,yjump: REAL;
    cmd: TEXT;
    mouse := NEW(FractalNetObj.MouseInfoStar);
  BEGIN
    LOCK mu DO
      xrange := (xmax - xmin);
      yrange := (ymax - ymin);
      xlow := ROUND((f.xmin-xmin)/xrange * FLOAT(sx));
      ylow := ROUND((f.ymin-ymin)/yrange * FLOAT(sy));
      xhigh := ROUND((f.xmax-xmin)/xrange * FLOAT(sx));
      yhigh := ROUND((f.ymax-ymin)/yrange * FLOAT(sy));
(*      Put ("Server received ReturnResults: (" & Fmt.Int(xlow) & ",");
      Put (Fmt.Int(ylow) & ") - (" & Fmt.Int(xhigh) & ",");
      Put (Fmt.Int(yhigh) & ")\n"); *) 
      FOR y := ylow TO yhigh DO
        FOR x := xlow TO xhigh-1 DO
            image_map[x][y] := pixmap[x-xlow][y-ylow];
        END;
      END; 
      numResults := numResults + 1;
      IF numResults = sy THEN
        image.width := sx;
        image.height := sy;
        image.data := image_map;
        FractalNetObj.DisplayFractal(image,crossHairs);
  
        REPEAT
          mouse := FractalNetObj.getMouseAction();
        UNTIL validCoords(mouse,FLOAT(sx),FLOAT(sy)); 

        xjump := (xrange / FLOAT(sx)) * (FLOAT(mouse.x) - (FLOAT(sy)/2.0));
        yjump := (yrange / FLOAT(sy)) * (FLOAT(mouse.y) - (FLOAT(sx)/2.0));

        ymin := ymin + xjump;
        ymax := ymax + xjump;
        xmin := xmin - yjump;
        xmax := xmax - yjump;

        IF mouse.button = 256 THEN
          xmin := xmin + (xrange * 0.05);
          xmax := xmax - (xrange * 0.05);
          ymin := ymin + (yrange * 0.05);
          ymax := ymax - (yrange * 0.05);
        ELSIF mouse.button = 1024 THEN
          xmin := xmin - (xrange * 0.05);
          xmax := xmax + (xrange * 0.05);
          ymin := ymin - (yrange * 0.05);
          ymax := ymax + (yrange * 0.05);
        ELSE
        END; 
        curY := 0;
        numResults := 0;
      END;
    END;
  END ReturnResults;

VAR
  cv := NEW(Thread.Condition);
  mutex := NEW(Thread.Mutex);
  myFractalNetObj := NEW(FractalNetObjImpl);
  mu := NEW(MUTEX);
  curY: INTEGER := 0;
  numResults: INTEGER := 0;
  crossHairs: INTEGER := 0;

  xmin: REAL := -2.50;
  xmax: REAL := 1.25;
  ymin: REAL := -1.25;
  ymax: REAL := 1.25;

  sx: INTEGER := 320;
  sy: INTEGER := 240;
  maxiter: INTEGER := 96;
  image_map: FractalNetObj.PixMap;
  image := NEW(FractalNetObj.ImageStar);

BEGIN

  WITH 
    pp = NEW(ParseParams.T).init(Stdio.stderr)
  DO
    TRY
      IF pp.keywordPresent("-crosshairs") THEN
        crossHairs := 1;
      END;
      IF pp.keywordPresent("-width") THEN
        sx := pp.getNextInt(1,1600);
      END;
      IF pp.keywordPresent("-height") THEN
        sy := pp.getNextInt(1,1200);
      END;
      pp.finish();
    EXCEPT
      ParseParams.Error =>
        Wr.PutText(Stdio.stderr, "Usage: server [-width #] [-height #] [-crosshairs]\n");
        Process.Exit(1);
    END;
  END;
  image_map := NEW(FractalNetObj.PixMap,sx,sy);

  Put("Fractal Server:\n\n");
  Put("Now you should start one or more clients using \"client [hostname]\"\n");
  Put("Use the mouse, left click to center and zoom in, right click to\n");
  Put("  center and zoom out.\n");
  FractalNetObj.InitUI(sx,sy);
  TRY
    LOCK mutex DO
      NetObj.Export("fractal", myFractalNetObj);
      Thread.Wait(mutex,cv); 
    END;
  EXCEPT
  | NetObj.Error (atom) =>
      WHILE atom # NIL DO
      Put ("***Error> " & Atom.ToText (atom.head) & "\n");
      atom := atom.tail;
      END;
  ELSE
    Put ("an unknown exception occured\n");
  END;
END Server.
